class Rack::Attack
  ### Allow Localhost ###
  # Always allow requests from localhost
  # (blocklist & throttles are skipped)
  safelist("allow from localhost") do |req|
    # Allow Access from localhost
    req.ip == "127.0.0.1" || req.ip == "::1"
  end

  blocklist("check authorization") do |req|
    # uncomment this line if you implement graphql_token validation
    # graphiql_token = ENV.fetch("GRAPHIQL_TOKEN")
    # is_graphiql_request = req.env["HTTP_GRAPHIQL_TOKEN"] == graphiql_token
    # next if !req.path.start_with?("/graphql") || is_graphiql_request
    next if !req.path.start_with?("/graphql")
    request_validator = GraphQLAuthorizer::Request.new(
      sig: req.env["HTTP_SIGNATURE"], # replace with Signature Header
      timestamp: req.env["HTTP_TIMESTAMP"] # replace with Time Stamp Header
    )
    next if request_validator.valid?
    error_message = request_validator.errors.to_sentence
    self.blocklisted_response = lambda do |_env|
      [
        401,
        { "Content-Type" => "application/json" },
        [{ errors: [{ message: error_message }] }.to_json]
      ]
    end
  end

  ### Throttle Spammy Clients ###

  # If any single client IP is making tons of requests, then they"re
  # probably malicious or a poorly-configured scraper. Either way, they
  # don"t deserve to hog all of the app server"s CPU. Cut them off!
  #
  # Note: If you"re serving assets through rack, those requests may be
  # counted by rack-attack and this throttle may be activated too
  # quickly. If so, enable the condition to exclude them from tracking.

  # Throttle all requests by IP (60rpm)
  #
  # Key: "rack::attack:#{Time.now.to_i/:period}:req/ip:#{req.ip}"
  throttle("req/ip", limit: 300, period: 5.minutes) do |req|
    req.ip unless req.path.start_with?("/assets")
  end

  ### Prevent Brute-Force Login Attacks ###

  # The most common brute-force login attack is a brute-force password
  # attack where an attacker simply tries a large number of emails and
  # passwords to see if any credentials match.
  #
  # Another common method of attack is to use a swarm of computers with
  # different IPs to try brute-forcing a password for a specific account.

  # Throttle POST requests to /login by IP address
  #
  # Key: "rack::attack:#{Time.now.to_i/:period}:logins/ip:#{req.ip}"
  throttle("cms login throttle ip", limit: 5, period: 20.seconds) do |req|
    req.ip if req.path == "users/sign_in" && req.post?
  end

  # Throttle POST requests to /login by email param
  #
  # Key: "rack::attack:#{Time.now.to_i/:period}:logins/email:#{req.email}"
  #
  # Note: This creates a problem where a malicious user could intentionally
  # throttle logins for another user and force their login requests to be
  # denied, but that"s not very common and shouldn"t happen to you. (Knock
  # on wood!)
  throttle("cms login throttle email", limit: 5, period: 20.seconds) do |req|
    if req.path == "users/sign_in" && req.post?
      # return the email if present, nil otherwise
      req.params["email"].presence
    end
  end
end
